#ifndef AK_CLIENT_H
#define AK_CLIENT_H

#include "cap.h"

/**
  * Class that provides a C compatible interface to communicate with
  * Akenti Server
  * 
  * @see AkentiServer
  *
  * @version 1.1 03/12/02
  * @author Mary Thompson
  */

#ifdef __cplusplus
extern "C" {
#endif

  /**
    *  Initializes the client interface according to the values in an
    *  akentiClient configuration file. 
    * 
    *  @param name of the client-side Akenti configuration file
    *    includes the name of the host on which the Akenti server runs 
    *    number of the port on which the Akenti server is running
    *    whether the server is using SSL connections or not.
    *    For SSL servers, it also provides the trusted CA certificate
    *    and the client cert and key if client-side authentication is required.
    *  @param status a returned string describing how the intitializion 
    *    proceeded
    *  @return 0 if the initialization was successful.
    */
  int akentiClientInit(const char *confFile, char **status);


  /**
    *  Given the DN of a user and the DN of the CA that issued this name and
    *  the name of the resource, returns the capability structure which
    *  contains the unconditional and conditional rights. This method
    *  sends a messaage to an Akenti server. It is callable from C code. 
    *
    *  Note: akentiClientInit must be called prior to this call.
    *
    *  @param subjectDN a string containing the DN of the user
    *        for whom we are checking access
    *  @param subjectCA a string containing the DN of the CA who issued the
    *        DN for whom we are checking access
    *  @param resource name of the resource for which access is checked
    *  @param status a returned error structure containing an error code and
    *     error string
    *  @param cap a returned pointer to the user's capability certificate;
    *     this pointer can be queried for unconditional and conditional
    *     rights plus contraints.
    *  @return  0 if the user is allowed some access
    *
    *  @see CapCert
    *  @see AKMessage, freeAkMessage
    */
  int remoteCheckAccess(const char *subjectDN, 
                        const char *subjectCA,
                        const char *resource, 
                        CapCert *cap, 
                        AKMessage *status);



  /**
    *  Given a string containing a PEM encoded x509 certificate for a user, 
    *  and the name of the resource, returns the capability structure which
    *  contains the unconditional and conditional rights. This method
    *  sends a messaage to an Akenti server. It is callable from C code. 
    *
    *  Note: akentiClientInit must be called prior to this call.
    *
    *  @param subjectCertString a string containing the PME encoded X.509 
    *        certificate for user for whom we are checking access
    *  @param resource name of the resource for which access is checked
    *  @param status a returned error structure containing an error code and
    *     error string
    *  @param cap a returned pointer to the user's capability certificate;
    *     this pointer can be queried for unconditional and conditional
    *     rights plus contraints.
    *  @return  0 if the user is allowed some access
    *
    *  @see CapCert
    *  @see AKMessage, freeAkMessage
    */
  int remoteCheckAccessByCertString(const char *subjectCert,
                                    const char *resource, CapCert *cap, 
                                    AKMessage *status);


  /**
    *  Given a file name which  contains a PEM encoded x509 certificate 
    *  or certificate chain for a user, 
    *  and the name of the resource, returns the capability structure which
    *  contains the unconditional and conditional rights.This method
    *  sends a messaage to an Akenti server. It is callable from C code. 
    *
    *  Note: akentiClientInit must be called prior to this call.
    *
    *  @param subjectCertString a string containing the PME encoded X.509 
    *        certificate for user for whom we are checking access
    *  @param resource name of the resource for which access is to be checked
    *  @param status a returned error structure containing an error code and
    *     error string
    *  @param cap a returned pointer to the user's capability certificate;
    *     this pointer can be queried for unconditional and conditional
    *     rights plus contraints.
    *  @return  0 if the user is allowed some access
    *
    *  @see CapCert
    *  @see AKMessage, freeAkMessage
    */
  int remoteCheckAccessByCertFile(const char *subjectCertFile,
                                  const char *resource, CapCert *cap, 
  	                          AKMessage *status);


#ifdef __cplusplus
}
#endif  
#endif
