#ifndef AKENTI_ENGINE_HPP
#define AKENTI_ENGINE_HPP

#include <string>
#include <set>
#include <map>

#include "LogInterface.hpp"
#include "PolicyContext.hpp"
#include "CapabilityCertificate.hpp"
#include "ExternalEvaluator.hpp"
#include "Properties.hpp"

using std::string;
using std::set;
using std::map;

namespace Akenti {

class CertificateChain;
class AKRuntime;
class AkentiPrincipal;
class AkentiMessage;
class Logger;

/**
  * Class that provides the methods to check a user's access to resource.
  * 
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */
class AkentiEngine : public LogInterface {

public:

  /**
    *  Constructor
    */
  AkentiEngine();

  /**
    * Destructor
    */
  ~AkentiEngine();
  
  /**
    * Called prior to a call to checkAccess to specify what actions
    * are to be checked for.
    *
    * @param desiredActions list of string actions that match some
    *        of the actions granted by the UseConditons for the resource
    * @see checkAccess
    */
  void setDesiredActions(const set<string>& desiredActions);

  /**
    * Called prior to a call to checkAccess to specify what attributes
    * and values are to be assumed for the user
    *
    * @param restrictedAttributes list of string attribute/value pairs that
    *     that will be assumed for the user on subsequent calls to checkAccess
    *
    * @see checkAccess
    */
  void setRestrictedAttributes(const multimap<string, string>& restrictedAttributes);

  /**
    * Sets an externalEvaluator that will be called by the authorization
    * module to evaluate any contraints of type SYSTEM or EXT_AUTH during
    * subsequent calls to checkAccess
    *
    * @param externalEvaluator a function that can be called to evaluate
    * UseConditions.
    *
    * @see AttributeInfo
    * @see ExternalEvaluator for an example of how to use an ExternalEvaluator
    */
  void setExternalEvaluator(const ExternalEvaluator& externalEvaluator);

  /**
    *  Given an authenticated principal name which consists of an x509 
    *  distinguished name for a user, and the x509 distinguished name of
    *  the Certificate Authority which issued the user's x509 cert, and
    *  the name of the resource, returns a capability certificate which
    *  can contain unconditional and conditional actions that the
    *  user has with respect to the resource.
    * <p>
    *  If required actions have been set by a previous call to 
    *  setDesiredActions, only those actions will be checked for. If
    *  a restricted set of attribute/value pairs have been set by a previous
    *  call to setDesiredAttributes, only those attributes will be checked
    *  and only the actions granted by those attributes will be returned.
    *
    *  @param resource  name of the resource for which access is to be checked
    *  @param principal user's DN, and DN of the CA that issued the user's cert
    *  @param cert      capability certificate if user is not denied access
    *  @param mesg      will have a response code and a description 
    *
    *  @return true if the user is not denied access.
    *  @see setDesiredActions
    *  @see setDesiredAttributes
    *  @see AkentiMessage
    */
  bool
  checkAccess(const string& resource,
              const AkentiPrincipal& principal,
              CapabilityCertificate& cert,
              AkentiMessage& mesg);
  /**
    *  Given an X509 certificate chain for a user  and
    *  the name of the resource, returns a capability certificate which
    *  can contain unconditional and conditional actions that the
    *  user has with respect to the resource.
    * <p>
    *  If required actions have been set by a previous call to 
    *  setDesiredActions, only those actions will be checked for. If
    *  a restricted set of attribute/value pairs have been set by a previous
    *  call to setDesiredAttributes, only those attributes will be checked
    *  and only the actions granted by those attributes will be returned.
    *
    *  @param resource  name of the resource for which access is to be checked
    *  @param principal user's DN, and DN of the CA that issued the user's cert
    *  @param cert      capability certificate if user is not denied access
    *  @param mesg      will have a response code and a description 
    *
    *  @return true if the user is not denied access.
    *  @see setDesiredActions
    *  @see setDesiredAttributes
    *  @see AkentiMessage
    */
  bool
  checkAccess(const string& resource,
              const CertificateChain& chain,
              CapabilityCertificate& cert,
              AkentiMessage& mesg);

  /**
    *
    */
  bool
  checkAccess(const string& resource,
              const AkentiPrincipal& principal,
              PolicyContext& context,
              AkentiMessage& mesg);

  /**
    */
  bool
  checkAccess(const string& resource,
              const CertificateChain& chain,
              PolicyContext& context,
              AkentiMessage& mesg);

  /**
    *  Loads a mapfile that maps user friendly names of resources  
    *  to the pathnames where the policy files for the resource are stored. 
    *
    *  This method should be called once at startup.
    *  <p>
    *  The filename of the resource map is given by the optional directive 
    *  ResourceMap <filename> in the Akenti Config file.
    */
  static void 
  loadResourceMap(const string& fileName);


  /**
    * Experimental: Collects and verifies policies and ucs.
    */
  bool checkPolicy(const string& resource,
                   PolicyContext& context, 
                   AkentiMessage& mesg);
  /**
    * Experimental: Is this <i>ap</i> an issuer for resource <i>resource</i>.
    */
  bool
  checkIssuer(const string& resource, const AkentiPrincipal& ap, AkentiMessage& mesg);

private:
 
  /**
    */
  bool 
  checkAccess(const string& resource,
              const AkentiPrincipal& principal,
              AKRuntime* akRuntime,
              AkentiMessage& mesg) const;
  bool
  collectAndVerifyPolicies(AKRuntime* akRuntime, AkentiMessage& mesg) const;

  bool
  collectAndVerifyUseConds(AKRuntime* akRuntime, AkentiMessage& mesg) const;

  bool
  doAuthorization(AKRuntime* akRuntime, 
                  AkentiMessage& mesg) const;

  CapabilityCertificate 
  generateCapabilityCert(const PolicyContext& context, long cacheInterval) const;

  /**
    *
    */
  Logger* initLogger(const string& id) const;

  /**
    */
  static Properties resourceMap;

  AKRuntime *akRuntime;

};

} // end of name space

#endif
