#ifndef AKENTI_INPUT_STREAM_HPP
#define AKENTI_INPUT_STREAM_HPP

#include <iostream>
#include <string>

using std::istream;
using std::string;

namespace Akenti {

class UtcTime;
class URL;
class DistinguishedName;
class AkentiObject;

/** 
  * An <i>AkentiInputStream</i> is used to deserialize objects 
  * from an ASCI string that was written using the conventions defined in  
  * <i>AkentiOutputStream</i>. This class provides input operations for
  * standard C++ types such as chars, ints, longs and strings as well as
  * Akenti-specific objects such as utctime, URL and Distinguished Names.
  *
  * Note that this object holds a reference to an underlying 
  * input stream and will only be valid as long as the underlying
  * stream is valid. For the standard C++ types, this class just wraps
  * the istream methods, so the standard conventions for reading from
  * C++ streams are followed.
  *
  * @see AkentiOutputStream
  * 
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai 
  * @author Abdelilah Essiari
  * @author Xiang Sun
  **/

class AkentiInputStream {

public:

  /**
    * Constructs this <i>AkentiInputStream</i> given a reference to 
    * an input stream specified by <i>is</i>.
    *
    * @param is istream the underlying input stream
    *
    */
  AkentiInputStream(istream& is);

  /**
    * Destructor.
    **/
  ~AkentiInputStream();

  /**
    * Reads a <i>char</i> from the underlying input stream.
    *
    * @param c char an out parameter
    * @return this AkentiInputStream for chaining 
    *
    * @exception IOException if reading from the stream fails 
    */ 
   AkentiInputStream& operator>>(char& c);

  /**
    * Reads a <i>int</i> from the underlying input stream.
    *
    * @param n int out parameter
    * @return this AkentiInputStream for chaining 
    * @exception IOException if reading from the stream fails 
    *
    * @see IOException
    */
   AkentiInputStream& operator>>(int& n);

  /**
    * Reads a <i>bool</i> from the underlying input stream.
    *
    * @param b bool an out parameter
    * @return this AkentiInputStream for chaining 
    * @exception IOException if reading from the stream fails 
    *
    * @see IOException
    */ 
   AkentiInputStream& operator>>(bool& b);

  /**
    * Reads a <i>long</i> from the underlying input stream.
    *
    * @param l long an out parameter
    * @return this AkentiInputStream for chaining 
    * @exception IOException if reading from the stream fails 
    *
    * @see IOException
    */
   AkentiInputStream& operator>>(long& l);

  /**
    * Reads a <i>double</i> from the underlying input stream.
    *
    * @param d double an out parameter
    * @return this AkentiInputStream for chaining 
    * @exception IOException if reading from the stream fails 
    *
    * @see IOException
    */
   AkentiInputStream& operator>>(double& d);

  /**
    * Reads one <i>char</i> using the istream::get method from the 
    * underlying input stream.
    * 
    * @return the char read  
    */
   char get();

  /**
    * Reads <i>len</i> chararacters from the underlying input stream
    * into the buffer specified by <i>buffer</i>. Uses the istream::read
    * method.
    *
    * @param buffer char* out parameter 
    * @param len number of chars to be read
    */
   AkentiInputStream& read(char* buffer, int len);
  
  /**
    * Reads a <i>string</i> from the underlying input stream. Implements
    * the convention that the stream representation of a single string consists
    * of ASCII characters terminated by a space unless the space is preceeded
    * by a '\'. In that case the space is considered to be part of the string,
    * and scanning continues until an non-escaped space is found.
    * 
    * @param s string out parameter
    * @return this AkentiInputStream for chaining 
    * @exception IOException if reading from the stream fails 
    *
    * @see IOException
    * @see AkentiOutputStream::write(string &s); 
   **/
   AkentiInputStream& operator>>(string& s);

  /**
    * Reads a <i>UtcTime</i> from the underlying input stream.
    *
    * @param utcTime UtcTime out parameter
    * @exception IOException if reading from the stream fails
    *
    * @see UtcTime for the for external format
    * @see IOException
   **/   
   AkentiInputStream& operator>>(UtcTime& utcTime);

  /**
    * Reads a <i>URL</i> from the underlying input stream.
    *
    * @param url URL out parameter
    * @return this AkentiInputStream for chaining 
    * @exception IOException if reading from the stream fails
    *
    * @see URL for the for external format
    * @see IOException
   **/   
   AkentiInputStream& operator>>(URL& utcTime);

  /**
    * Reads a <i>DistinguishedName</i> from the underlying input stream.
    *
    * @param dn DistinguishedName out parameter
    * @return this AkentiInputStream for chaining 
    * @exception IOException if reading from the stream fails
    *
    * @see DistinguishedName for the for external format
    * @see IOException
   **/   
   AkentiInputStream& operator>>(DistinguishedName& dn);

  /**
    * Read an <i>AkentiObject</i> from the underlying input stream.
    * Calls the readObject method on the object handing it the 
    * AkentiInputStream.
    *
    * @param obj AkentiObject out parameter
    * @return this AkentiInputStream for chaining 
    * @exception IOException if reading from the stream fails
    *
    * @see AkentiObject
    */
   AkentiInputStream& operator>>(AkentiObject& obj);

private:

  istream& is;         // underlying stream

};

} // end of name space

#endif
