#ifndef AKENTI_OUTPUT_STREAM_HPP
#define AKENTI_OUTPUT_STREAM_HPP

#include <iostream>
#include <string>

using std::ostream;
using std::string;

namespace Akenti {

class UtcTime;
class URL;
class DistinguishedName;
class AkentiObject;

/**
  * An <i>AkentiOutputStream</i> object is used to serialize all
  * types of objects using a simple scheme to write the objects'
  * string reprsentations. For standard C++ types such as int, long, char
  * this class just wraps the ostream methods.
  *
  * For string objects containing blank spaces those
  * spaces are escaped by preceeding them 
  * by a back slash. Escaping the blank space allows the deserializing
  * process to distinguish between blank spaces that are part of the
  * object being deserialized and the blank spaces used as delimiters
  * between elements. Only the object is written to the ostream by these
  * methods. The delimiting blank space between the ojbects must be written
  * by the caller of this class.
  *
  * Note:
  *
  *   Blank spaces at the beginning and at the end of an object's 
  *   representation are ignored. Those in the middle are escaped 
  *   and if there is a sequence of blanks in the middle, 
  *   only one is escaped and the others are ignored.
  *
  * Example1:
  *   
  * "/C=US/CN=John Doe" will be written as "/C=US/CN=John\ Doe"
  *
  * Example2:
  *
  *   Deserializing "Hello\ John Bye\ John" will result in
  *   two strings "Hello John" and "Bye John"
  *  
  * It is recommended to use an <i>AkentiInputStream</i> object 
  * to deserialize objects.
  *
  * @see AkentiInputStream 
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  * @author Xiang Sun
  **/

class AkentiOutputStream { 

public:

  /**
    * Constructs this object given an <i>ostream</i> object
    * specified by <i>os</i>. Note that the AkentiOutputStream is only
    * valid as long as the underlying ostream is valid.
    *
    * @param os ostream the underlying output stream
    */
  AkentiOutputStream(ostream& os);

  /**
    */
  AkentiOutputStream& write(const char *ptr, int n);

  /**
    * Writes a <i>char</i> to the underlying output stream. 
    *
    * @param c char to be written
    * @return this AkentiOutputStream for chaining 
    * @exception IOException if writing fails
    *
    * @see IOException
    */
   AkentiOutputStream& operator<<(char c);

  /**
    * Writes an <i>int</i> to the underlying output stream. 
    *
    * @param n int to be written
    * @return this AkentiOutputStream for chaining 
    * @exception IOException if writing fails
    *
    * @see IOException
    */
   AkentiOutputStream& operator<<(int n);

  /**
    * Writes a <i>bool</i> to the underlying output stream. 
    *
    * @param b bool to be written
    * @return this AkentiOutputStream for chaining 
    * @exception IOException if writing fails
    *
    * @see IOException
    */
   AkentiOutputStream& operator<<(bool b);

  /**
    * Writes a <i>long</i> to the underlying output stream.
    *
    * @param l long to be written
    * @return this AkentiOutputStream for chaining
    * @exception IOException if writing fails
    *
    * @see IOException
    */
   AkentiOutputStream& operator<<(long l);

  /**
    * Writes a <i>double</i> to the underlying output stream.
    *
    * @param d double to be written
    * @return this AkentiOutputStream for chaining
    * @exception IOException if writing fails
    *
    * @see IOException
    */
   AkentiOutputStream& operator<<(double d);

  /**
    * Writes a <i>string</i> to the underlying output stream.
    *
    * Blank spaces at the beginning and at the end of the string are ignored.
    * Blanks in the middle are escaped and if there is a sequence of blanks
    * in the middle, only one is escaped and the others are ignored.
    *
    * Examples:
    *
    *   "aa bb"       => "aa\\ bb"
    *   " aa bb "     => "aa\\ bb"
    *   "aa  bb"      => "aa\\ bb"
    *
    * @param s  string to be written 
    * @return this AkentiOutputStream for chaining
    * @exception IOException if writing fails
    **/
   AkentiOutputStream& operator<<(const string& s);

  /**
    * Writes a <i>UtcTime</i> to the underlying output stream.
    *
    * @param time UtcTime to be written
    * @return this AkentiOutputStream for chaining
    * @exception IOException if writing fails
    *
    * @see UtcTime for the external format that is used
    * @see IOException
    */
   AkentiOutputStream& operator<<(const UtcTime& time);

  /**
    * Writes a <i>URL</i> to the underlying output stream.
    *
    * @param url URL to be written
    * @return this AkentiOutputStream for chaining
    * @exception IOException if writing fails
    *
    * @see URL for the external format that is used
    * @see IOException
    */
   AkentiOutputStream& operator<<(const URL& url);

  /**
    * Writes a <i>DistinguishedName</i> to the underlying output stream.
    *
    * @param dn DistinguishedName to be written
    * @return this AkentiOutputStream for chaining
    * @exception IOException if writing fails
    *
    * @see DistinguishedName for the external format that is used
    * @see IOException
    */
   AkentiOutputStream& operator<<(const DistinguishedName& dn);
  
  /**
    * Writes an <i>AkentiObject</i> to the underlying output stream.
    * Calls the writeObject method on the object handing it the
    * AkentiOutputStream.
    *
    * @param obj AkentiObject to be written 
    * @return this AkentiOutputStream for chaining
    * @exception IOException if writing fails
    *
    * @see AkentiObject 
    * @see IOException
    **/   
   AkentiOutputStream& operator<<(const AkentiObject& obj);

private:

  ostream& os;

};

} // end of name space

#endif
