#ifndef AK_BASE_64_IMPL
#define AK_BASE_64_IMPL

#include <string>
#include "Error.hpp"

using std::string;

/**
  * This abstract class provides part of the functionality
  * of the Base 64 encoding/decoding.
  * The rest of the functionality is provided by the subclass 
  * implementors.
  *
  * Typically the clients will not deal with an instance of this    
  * class directly, instead they interact with the <i>Base64</i> 
  * reference-count pointer.
  *
  * <pre>
  * <b>Usage</b>
  *  try {
  *    string data = "1234567890";
  *    string enc = Base64::encode(data);
  *    string dec = Base64::decode(enc);
  *
  *    if (dec == enc) {
  *      cout << "** Base64 Encoding/Decoding WORKED **";
  *    } else {
  *      cout << "!! Base64 Encoding/Decoding FAILED !!";
  *    }
  *  } catch(Exception& ex) {.....}
  *</pre>
  *
  * @see Base64
  * @see Provider
  * @see ProviderImpl#getProvider()
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  * @author Xiang Sun
  */

namespace Akenti {

class Base64Impl {

public:

  /**
    * Destroys this object  
    */
  virtual ~Base64Impl() {
  }

  /**
    * Initializes the engine to the encoding state.
    *
    * @see doEncodeInit()
    */
  void encodeInit() {
    state = ENCODING;
    doEncodeInit();
  }

  /**
    * Updates the encoding engine with data.
    *
    * @param data string used to update the engine
    * @exception  Error <i>encodeInit()</i> has not been called
    * @exception  Error if the input buffer is too short
    * @see encodeInit
    * @see encodeUpdate(string, int, int)
    */
  void encodeUpdate(const string& data){
    encodeUpdate(data, 0, data.length()); 
  }

  /**
    * Updates the encoding engine with data.
    *
    * @param data string used to update the engine
    * @param offset int index in data to start reading bytes from
    * @param len int number of bytes used to update the engine
    * @exception  Error <i>encodeInit()</i> has not been called
    * @exception  Error if the input buffer is too short
    * @see encodeInit
    * @see doEncodeUpdate(string, int, int)
    */
  void encodeUpdate(const string& data, int offset, int len){
    if (state != ENCODING) {
      throw Error("Base64Impl:encodeUpdate:illegal state");
    } else if (data.length() - offset < len) {
      throw Error("Base64Impl:encodeUpdate:input buffer too short");
    }

    doEncodeUpdate(data,offset,len); 
  }
 
  /**
    * Completes the encoding of the data that has been processed
    * through the engine. Resets engine state to UNINITIALIZED. 
    * 
    * @return the Base64 encoding 
    * @exception  Error if <i>encodeInit()</i> has not been called
    * @see doEncode()
    */ 
  string encode() {
    if (state != ENCODING) {
      throw Error("Base64Impl:encode:illegal operation");
    }

    string enc = doEncode();
    
    state = UNINITIALIZED;
    return enc;
  }

  /**
    * Initializes the engine to the decoding state.
    *
    * @see doDecodeInit()
    */
  void decodeInit() {
    state = DECODING;
    doDecodeInit();
  }
  

  /**
    * Updates the decoding engine with data.
    *
    * @param data string used to update the engine
    * @exception  Error <i>decodeInit()</i> has not been called
    * @exception  Error if the input buffer is too short
    * @exception SecurityException if decoding fails 
    * @see decodeInit
    * @see decodeUpdate(string, int, int)
    */
  void decodeUpdate(const string& data){ 
    decodeUpdate(data, 0, data.length()); 
  }

  /**
    * Updates the decoding engine with data.
    *
    * @param data string used to update the engine
    * @param offset int index in data to start reading bytes from
    * @param len int number of bytes used to update the engine
    * @exception  Error <i>decodeInit()</i> has not been called
    * @exception  Error if the input buffer is too short
    * @exception SecurityException if decoding fails 
    * @see decodeInit
    * @see doDecodeUpdate(string, int, int)
    */
  void decodeUpdate(const string& data, int offset, int len){ 
    if (state != DECODING) {
      throw Error("Base64Impl:decodeUpdate:illegal operation");
    } else if(data.length() - offset < len) {
      throw Error("Base64Impl:decodeUpdate:input buffer too short"); 
    }

    doDecodeUpdate(data,offset,len);
  }

  /** 
    * Completes the decoding of the data that has been processed
    * through the engine. Resets the engine state to UNINITIALIZED.
    *
    * @return the Base64 decoding
    * @exception  Error if <i>decodeInit()</i> has not been called
    * @exception SecurityException if decoding fails 
    * @see doDecode()
    */
  string decode() {
    if (state != DECODING) {
      throw Error("Base64Impl:decode:illegal operation");
    }
    
    string data = doDecode();

    state = UNINITIALIZED;
    return data;
  }
  
protected:

  /**
    * Constructs this object in the uninitalized state.
    * one must call <i>encodeInit()</i> to change to <i>ENCODING</i>
    * state for encoding purposes. 
    * one must call <i>decodeInit()</i> to change to <i>DECODING</i> 
    * state for decoding purposes.
    *
    * @see encodeInit()
    * @see decodeInit()
    */
  Base64Impl() : state(UNINITIALIZED) {
  }
  
  /**
    * Initilizes the engine to the encoding state.
    */
  virtual void doEncodeInit() = 0; 

  /**
    * Initilizes the engine to the decoding state.
    */
  virtual void doDecodeInit() = 0;

  /**
    * Completes the encoding of the data that has been processed
    * through the engine.
    */  
  virtual string doEncode() = 0; 

  /**
    * Completes the decoding of the data that has been processed
    * through the engine.
    *
    * @exception SecurityException if decoding fails 
    */
  virtual string doDecode() = 0;

  /**
    * Updates the encoding engine with data.
    *
    * @param data string used to update the engine
    * @param offset int index in data to start reading bytes from
    * @param len int number of bytes used to update the engine
    */
  virtual void doEncodeUpdate(const string& data, int offset, int len) = 0;

  /**
    * Updates the decoding engine with encoded string.
    *
    * @param enc string used to update the engine
    * @param offset int index in data to start reading bytes from
    * @param len int number of bytes used to update the engine
    * @exception SecurityException if decoding fails 
    */
  virtual void doDecodeUpdate(const string& enc, int offset, int len) = 0;

private:

  enum State { UNINITIALIZED, ENCODING, DECODING } state;

};

} // end of namespace Akenti

#endif

