#ifndef CERTIFICATE_RETRIEVAL_FACTORY_HPP
#define CERTIFICATE_RETRIEVAL_FACTORY_HPP

#include <string>
#include <map>
#include "ref_ptr.hpp"
#include "CertificateRetrievalImpl.hpp"

using std::string;
using std::map;
using namespace Akenti;

namespace Akenti {

class URL;
class Directory;

/**
  * Class <code>CertificateRetrieval</code> serves as the wrapper class
  * for <code>CertificateRetrievalImpl</code> objects. It has a
  * <i>reference-count</i> for the implementation object that it holds.
  *
  * @type CertificateRetrieval 
  *
  * @see ref_ptr
  * @see CertificateRetrievalImpl 
  */ 
typedef ref_ptr<CertificateRetrievalImpl> CertificateRetrieval;

/**
  * Class to manage CertificatRetrieval instances.
  * 
  * Note:
  *
  * In a multithreaded program: <BR>
  *
  *    1) Install custom made <i>CertificateRetrievalImpl</i> objects at 
  *       startup. <BR>
  *       
  *    2) The following static methods are thread safe.<BR>
  *
  *       <i>getCertificates(...)</i> and <i>getAkentiCertificate(...)</i><BR>
  *
  * @see CertificateRetrievalImpl
  * @see CertificateRetrievalFactory
  * @see CertificateRetrieval
  * @see Certificate
  * @see AkentiCertificate
  * @see URL
  * @see Directory
  * 
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

class CertificateRetrievalFactory {

public:

  /**
    */
  ~CertificateRetrievalFactory();

  /**
    */
  void 
  addCertificateRetrieval(const string& protocol,
                          CertificateRetrievalImpl* cri);

  /**
    */
  const CertificateRetrieval&
  getCertificateRetrieval(const string& protocol);

  /**
    */
  static CertificateRetrievalFactory&
  getCertificateRetrievalFactory();

  /**
    */
  static vector<Certificate>
  getCertificates(const URL& url,
                  map<string, string>& args);

  /**
    */
  static vector<Certificate>
  getCertificates(const Directory& directory,
                  map<string, string>& args);

  /**
    */
  static vector<AkentiCertificate>
  getAkentiCertificates(const Directory& directory,
                        map<string, string>& args);

  /**
    */
  static vector<AkentiCertificate>
  getAkentiCertificates(const URL& url,
                        map<string, string>& args);

private:

  CertificateRetrievalFactory();

  CertificateRetrievalFactory(const CertificateRetrievalFactory&);

  void operator=(const CertificateRetrievalFactory&);

  map<string, CertificateRetrieval> crs;
  
  static CertificateRetrievalFactory fac;

};

} // end of name space

#endif
