#ifndef CHECK_POL_COLLECTOR_HPP
#define CHECK_POL_COLLECTOR_HPP

#include <string>
#include <vector>
#include "Policy.hpp"
#include "AkentiMessage.hpp"
#include "LogInterface.hpp"

#include "CertInfoCollector.hpp"

using std::string;
using std::vector;
using namespace Akenti;

namespace Akenti {

class AKRuntime;
class AkentiCertificate;

/**
  * A <i>CheckPolCertCollector</i> collects and verifies <i>Policy</i> certificates. 
  *
  * <pre>
  *
  * Usage:
  *
  *     akRuntime->setResource ("resource");
  *     CheckPolCertCollector policyCollector(akRuntime, certInfoCol);
  *
  *     if (!policyCollector.collectPolicies()) {
  *       exit(0); 
  *     } 
  * 
  *     if (!policyCollector.verifyPolcies()) {
  *       exit(0);
  *     }
  *
  *     ...
  *
  * </pre>
  *
  * @see AKRuntime
  * @see Policy 
  * @see AkentiMessage 
  *
  * @version 1.1 04/05/19
  * @author Sushila Subramanian
  * @author Abdelilah Essiari
  * @author Srilekha Mudumbai 
  */

class CheckPolCertCollector : public LogInterface {
 
public:

  /**
    * Constructs this object. 
    *
    */
  CheckPolCertCollector(AKRuntime* akRuntime);

  /**
    * Destroys this object.
    */
  ~CheckPolCertCollector();
  
  /**
    * Collects all the policies, including ones that werent used or
    * had an error.
    */
  bool collectPolicies();

  /**
    * Verifies the collected policies that were used.
    */
  bool verifyPolicies();

  /**
    * Retruns status info.
    *
    * @see AkentiMessage
    */
  const AkentiMessage& getAkentiMessage() const;

private:

  /**
    * Returns true if the policy certificate verifies. 
    *
    * That is the policy certificate has not expired, has been issued 
    * by an issuer that belongs to one of the groups specified in the 
    * policy itself, and is found to be trusted.
    * 
    */
  bool verifyPolicy(const Policy& policy);

  /**
    *
    * Parses a resource name, whose components may be separated by /'s 
    * as in the case of hierarchical resources, into a list. 
    *
    * Exmaple: LBL/test/a results in (LBL, LBL/test, LBL/test/a).
    *
    * Conditions on the name of resources:
    *
    *    Must not be empty
    *    Must not contain blank spaces
    *    Must not end with a / 
    *
    * @return true if successful,
    *         false othewrwise
    */
  bool 
  parseResource(const string& resource, vector<string>& resources) const;

  /**
    * Reads certificate from a file.
    */
  PolicyCertificate
  readPolicyFile(const string& encType, const string& fileName, string& mesg) const;

private:

  AKRuntime* akRuntime;

  AkentiMessage message;

}; // end class

} // end of name space

#endif /* CHECK_POL_CERT_COLLECTOR */
