#ifndef EXTERNAL_EVALUATOR_IMPL_HPP
#define EXTERNAL_EVALUATOR_IMPL_HPP

#include "TokenType.hpp"
#include "AttributeInfo.hpp"
#include "CertificateVerifier.hpp"
#include "PolicyContext.hpp"
#include "CertificateCache.hpp"

namespace Akenti {

class AKRuntime;

/**
  * Subclass implementors should overide the virtual public
  * methods to help the engine evaluate 
  * 
  * @see ExternalEvaluator
  * @see AKRuntime
  * @version 1.1 03/04/2003
  * @author Abdelilah Essiari
  */

class ExternalEvaluatorImpl {

public:
  
  /**
    * Constructor.
    */
  ExternalEvaluatorImpl();

  /**
    * Destructor.
    */
  virtual ~ExternalEvaluatorImpl();

  /**
    * Returns <i>true</i> if this evaluator knows how to 
    * handle the <i>SYSTEM attribute</i> specified in the
    * argument.
    *
    * @see evaluateSystemAttribute(TokenType, const AttributeInfo&)
    */
  virtual bool canHandleSystemAttribute(const string& attribute);

  /**
    * Returns <i>true</i> if (attrInfo.getAttribute() op  attrInfo.getValue())
    *
    * Note: the engine will call this method only if 
    *      canHandleSystemAttribute<attrInfo.getAttribute() returned true.
    *
    * @see AttributeInfo 
    * @see TokenType
    */
  virtual bool evaluateSystemAttribute(TokenType op, const AttributeInfo& attrInfo);

  /**
    * Returns <i>true</i> if this evaluator knows how to 
    * handle the <i>EXT_AUTH attribute</i> specified in the
    * argument.
    *
    * @see evaluateExternalAttribute(TokenType, const AttributeInfo&)
    */
  virtual bool canHandleExternalAttribute(const string& attribute);

  /**
    * Returns <i>true</i> if (attrInfo.getAttribute() op  attrInfo.getValue())
    *
    * Note: the engine will call this method only if 
    *        canHandleHandleExternalAttribute<attrInfo.getAttribute()> returned true.
    *
    * @see AttributeInfo 
    * @see TokenType
    */
  virtual bool evaluateExternalAttribute(TokenType op, const AttributeInfo& attrInfo);

  /**
    * Returns the cache Interval.
    *   If return value > 0
    *     The engine may not cache the capability certificate
    *     but if it does it will cache it for no longer than the cache Interval. 
    * 
    *   Note: a return value < 0 implies the external evaluator does not care.
    */
  int getCacheInterval() const;

protected:

  /**
    * Allows the external evaluator to suggest whether it likes 
    * the capability certificate to be cached. A negative value
    * implies the evaluator does not care.
    */
  void setCacheInterval(int numOfseconds);

  /**
    * @see PolicyContext
    */
  const PolicyContext& getPolicyContext();

  /**
    * @see CertificateVerifier 
    */
  CertificateVerifier& getCertificateVerifier();

  /**
    *
    * @see CertificateCache 
    */
  CertificateCache& getCertificateCache();

private:

  friend class ExternalEvaluator;

  /**
    */
  AKRuntime* getAKRuntime();

  /**
    * @see AKRuntime
    */
  void setAKRuntime(AKRuntime* akRuntime);

  /** 
    */
  AKRuntime* akRuntime;
 
  /**
    */
  int cacheInterval;
};

}

#endif

