#ifndef AK_KEY_STORE_HPP
#define AK_KEY_STORE_HPP 

#include "Provider.hpp"
#include "KeyStoreImpl.hpp"
#include "ref_ptr.hpp"

namespace Akenti {

/**
  * A <i>KeyStore</i> is a <b>reference-count</b> pointer that wraps
  * a <i>KeyStoreImpl</i>. 
  *
  * <PRE>
  *
  * <b>Usage 1</b>
  *
  *    const KeyStore& st =  KeyStore::getInstance("pkcs12");
  *
  *    st->load("someFileName", "somePassword");
  *
  * </PRE>
  *
  * @see KeyStoreImpl 
  * @see Provider
  * @see ref_ptr
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

class KeyStore : public ref_ptr<KeyStoreImpl> {

public:

  /**
    * Constructs a <i>KeyStore</i> given a <i>KeyStoreImpl</i>.
    *
    * @see KeyStoreImpl
    * @see ref_ptr
    */
  KeyStore(KeyStoreImpl *impl) : ref_ptr<KeyStoreImpl>(impl) {
  }

  /**
    * Destroys this object.
    * 
    * @see ref_ptr
    */
  ~KeyStore() {
  }

  /**
    * Returns the <i>KeyStore</i> from the installed <i>Provider</i>.
    * 
    * @see SecurityManager#getProvider()
    * @see ProviderImpl#getKeyStoreImpl(const string&)
    */
  static KeyStore getInstance(const string& type) {
    Provider& po = Provider::getProvider();
    
    return po->getKeyStore(type);
  }

};

} // end of namespace Akenti

#endif
