#ifndef AK_LOGGER_HPP
#define AK_LOGGER_HPP

#include <string>
#include "Properties.hpp"

using std::string;


namespace Akenti {

/** 
  * <i>Logger</i>.
  *
  * @version 1.1 05/24/01
  * @author Mary Thompson
  * @author Gary Hoo
  * @author Abdelilah Essiari
  */

class Logger {

public:
  
  /**
    * Destroys this object.
    */
  virtual ~Logger();

  /**
    * Returns <i>true</i> if init has been called.
    *
    * @return true if already initialized.
    */
  static bool hasBeenInitialized();

  /**
    * Returns a <i>Logger</i> given an identity string specified by <i>id</i>.
    * This identity string will be included in all the subsequent logs and,
    * if possible, it should indicate who is making the request for access
    * i.e. the remote host that the request comes from and/or the 
    * distinguished name of the requestor. This <i>Logger</i> must be freed 
    * by the caller.
    *
    * We support 4 logging destinations server, file, stderr, stdout.
    * The destination parameters are loaded from the system properties.
    * The default is stderr.
    *
    * Note <i>NULL</i> is returned if we are unable to open logs 
    * destination.
    * 
    * @param id identity string that will be included in each log message
    * @param message string out parameter that stores the outcome of init 
    * @exception Error if logger has been initialized or bad memory allocation
    *
    * @return a logger that must be freed by the caller. 
    * @see Properties#getSystemProperties()
    */
  static Logger* init(const string& id, string& message);

  /**
    */
  static Logger* init(const string& id, const Properties& props, string& message);

  /**
    * Writes this log to the destination provided that
    * <i>level</i> is less than or equal to the logging level
    * specified in the config file. 
    *
    * Note: No logs will be writen if a successful call
    * to <i>init(...)</i> has not been made. 
    *
    * @param level int logging level of this message 
    * @param keyword string  application-specific event label
    * @param message string message to be logged
    *
    * @see Logger#setLogLevel()
    */
  static int write(int level, const string& key, const string& message);

  /**
    * Sets the logging level for this <i>Logger</i> to <i>level</i>. 
    * All <i>write(...)</i> calls include as their first argument a logging 
    * level. Only writes with a logging level less than <i>level</i> 
    * will be logged. This allows a program to change at runtime how much 
    * logging to perform
    *
    * @param level int the new logging level for this logger
    * @return the old logging level
    *
    * @see Logger#write(int, const string&, const string&).  
    */
  static int setLevel(int level);

  /**
    */
  static void flush();

protected:

  /**
    */
  virtual int doWrite(int, const string&, const string&, const string&);

  /**
    */
  virtual int doSetLevel(int);

  /**
    */
  virtual void doFlush();

};

  /** 
    * log error messages only. 
    */
  static const int LOG_AUD   = 1;

  /** 
    * log monitoring and error messages. 
    */
  static const int LOG_MON   = 2;  

  /** 
    * log first debug level, monitoring and error messages. 
    */
  static const int LOG_L3    = 3;            

  /** 
    * log second and first debug level, monitoring and error messages. 
    */
  static const int LOG_L4    = 4; 

  /** 
    * log third, 2nd and 1st debug level, monitoring and error messages. 
    */
  static const int LOG_L5    = 5;           


} // end of namespace Akenti

#endif
