#ifndef POLICY_AUTHORIZER_HPP
#define POLICY_AUTHORIZER_HPP

#include <vector>
#include <string>
#include <set>
#include "AkentiMessage.hpp" 
#include "UseCondCertificate.hpp" 
#include "Policy.hpp" 
#include "LogInterface.hpp"

using std::string;
using std::vector;
using std::set;

namespace Akenti {

class Evaluator;
class AKRuntime;
class AkentiCertificate; 
class ConditionalActions; 

/**
  * A <i>PolicyAuthorizer</i> evalautes the <i>UseCondition</i>
  * certifcates and generates <i>Capability</i> if at least 
  * one <i>UseCondition</i> certificate has been satisfied  
  * or partially satisfied and if no <i>UseCondition</i> certificate 
  * with the enable flag set is not satisfied.
  *
  * @see AKRuntime
  * @see Evaluator 
  *
  * @version 1.1 00/05/01
  * @author Abdlelilah Essiari
  * @author Srilekha Mudumbai
  */

class PolicyAuthorizer : public LogInterface {
  
public:

  /**
    * Constructs this object.
    *
    */
  PolicyAuthorizer(AKRuntime* akRuntime);
  
  /**
    * Destroys this object.
    */
  ~PolicyAuthorizer();

  /**
    * Determines the allowable actions.
    * <pre>
    * Algorithm to determine these actions:
    *
    *   For each policy:
    *     Evaluate the UseCondition certificates. 
    *       For each UseCondition:
    *          If a UseCondition certificate evaluates to <i>false</i>
    *            and enable is <i>true</i> access is denied.
    *          If a UseCondition certificate evaluates to <i>true</i> 
    *            we just add the actions specified by that UseCondition.
    *          Otherwise we do nothing.
    *
    * Algorithm to evaluate a <i>UseCondition</i>:
    *
    *   The boolean expression specified in the <i>UseCondition</i>
    *   is evaluated using short-circuit evaluation. Therefore not
    *   all attribute/value pairs are considered.
    *   For each attribute/value pair, we have two cases:
    *     a) X509 
    *        First we check if the CA of the <i>AkentiPrincipal</i> 
    *        is one of the CAs that can attest to this attribute value pair.
    *        Second we check if the <i>DistinguishedName</i> contains the  
    *        attribute/value pair. 
    *     b) GENERIC:
    *        This involves using attribute certificates. 
    *        The collection of attribute certificates is essenatially the 
    *        same as the collection of X509 identity certificates. See
    *        CertificateVerifier. The verification has an additional test
    *        which makes sure that the issuer of the attribute certificate
    *        is one of the issuers for this attribute/value pair. 
    * </pre>
    */
  bool authorize();

  /**
    */
  const AkentiMessage& getAkentiMessage() const;

private:

  int 
  authorize(Evaluator& evaluator, const vector<UseCondCertificate>& ucCerts);

  AKRuntime* akRuntime;
  
  AkentiMessage message;

  set<string> minimalActions;
  set<string> unconditionalActions;
  vector<ConditionalActions> condActionsList;
};

} // end of name space

#endif
