#ifndef POLICY_CHECKER_HPP
#define POLICY_CHECKER_HPP

#include <string>
#include "PolicyContext.hpp"
#include "LogInterface.hpp"
#include "Properties.hpp"
#include "CertInfoCollector.hpp"

using std::string; 

namespace Akenti {

class AKRuntime;
class Logger;
class AkentiMessage;

/** 
  * A <i>PolicyChecker</i> goes through all the <i>Policy</i> 
  * certificates available for the given resource, and all the 
  * <i>Use Condition</i> certificates associated with the 
  * <i>Policy</i> certificates collected so far. In all cases
  * it collects an error message if the certificate has a problem
  * or an "OK" message if it was fine. It also collects information
  * on whether the certificates were used or not.
  * This file is based on the original PolicyChecker written by
  * Abdelilah Essiari and Srilekha Mudumbai
  * 
  * see @AKRuntime
  * see @PolicyContext
  * see CertInfoCollector
  *
  * @version 1.0  04/05/20
  * @author Sushila R. Subramanian
  */
  

class PolicyChecker : public LogInterface {

public:

  /** 
    * Constructs this object 
    */

  PolicyChecker();

  /**
    * Destroys this object 
    */
  ~PolicyChecker();

  /** 
    * Collects Policy certificates associated with a resources; collects
    * AkentiMessages that state if the certificates were verified, or 
    * had some error associated with them; Collects <i>Use Condition</i>
    * certificates associated with the above Policy certificates - and 
    * error or "OK" messages associated with them. It gathers all this 
    * information in the <i>CertInfoCollector</i> object.
    *
    * @see PolicyContext
    * @see AkentiMessage
    * 
    */

  bool checkPolicy(const string& resource, CertInfoCollector &cinfoCol, 
                        AkentiMessage& mesg);


private:

  bool collectAndVerifyPolicies(AKRuntime* akRuntime, AkentiMessage& mesg) const;

  bool collectAndVerifyUseConds(AKRuntime* akRuntime, AkentiMessage& mesg) const;

  bool verifyCAS(AKRuntime* akRuntime, AkentiMessage& mesg) const;

  bool verifyUCIssuers(AKRuntime* akRuntime, AkentiMessage& mesg) const;

  Logger* initLogger(const string& id) const;

  void loadResourceMap(const string& fileName);
  
  AKRuntime* akRuntime;

  static Properties resourceMap;
}; // class ends

} // namespace ends

#endif /* POLICY_CHECKER_HPP */
