#ifndef AK_SSL_SOCKET_HPP
#define AK_SSL_SOCKET_HPP

#include "SSLContext.hpp"
#include "Socket.hpp"
#include "SecureSocketImpl.hpp"
#include "Provider.hpp"

namespace Akenti {

/** 
  * <i>SSLSocket</i> class - derived from <i>Socket</i> but has a 
  * <i>SSLcontext</i> associated with it. Inherits most of its methods 
  * from <i>Socket</i> 
  * i.e. setting the host, setting/getting the port, connection and binding. 
  *
  * @see SSLContext
  * @see Socket
  * @see SecureSocketImpl
  * @see Provider
  *  
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */ 

class SSLSocket : public Socket {
public:

  /**
    * Constructs a SSLSocket from a SecureSocketImpl.
    *
    * @param SecureSocketImpl probably created by the newInstance method
    *        of the concrete SecureSocketImpl.
    */
  SSLSocket(SecureSocketImpl* impl = NULL) : Socket(impl) {
  }

  /**
    * Constructs a client SSLSocket from the Provider default SSLClientContext
    * and  default SecureSocketImpl.
    *
    * @param host to which the socket will connect.
    * @param port to which the socket will connect.
    * @see Provider
    */
  SSLSocket(const string& host, int port) {
    Provider& po = Provider::getProvider();
    const SSLContext& ctx = SSLContext::getDefaultSSLClientContext();

    impl = po->createSecureSocketImpl(ctx);
    impl->setHost(host);
    impl->setPort(port);
  }

  /**
    * Constructs a client SSLSocket from a Server SSLContext and the Provider
    * default SecureSocketImpl.
    *
    * @param ctx a client SSLContext
    * @param  host host to which the socket will connect.
    * @param  port port to which the socket will connect.
    * @exception Error if the input SSLcontext is not a client context.
    * @see Provider
    */
  SSLSocket(const SSLContext& ctx, const string& host, int port) {
    if (!ctx->isClientContext()) {
      throw Error("SSLSocket:constructor:invalid context");
    }

    Provider& po = Provider::getProvider();

    impl = po->createSecureSocketImpl(ctx);
    impl->setHost(host);
    impl->setPort(port);
  }

  /**
    * Destroys this object.
    */
  ~SSLSocket() {
  }

  /**
    * Return the security  context associated with this socket.
    *
    * @return the SSLContext associated with this socket.
    */
  const SSLContext& getSSLContext() {
    SecureSocketImpl* dimpl = (SecureSocketImpl*) impl.get();

    return dimpl->getSSLContext();
  }

  Certificate getPeerCertificate() {
    SecureSocketImpl* dimpl = (SecureSocketImpl*) impl.get();

    return dimpl->getPeerCertificate();
  }

private:
   
  friend class Socket;

  typedef SecureSocketImpl* ImplType;
};

} // end of namespace Akenti

#endif

