#ifndef AK_RUNTIME
#define AK_RUNTIME

#include "AKCertHolder.hpp"
#include "Policy.hpp"
#include "PolicyContext.hpp"
#include "CertificateCache.hpp"
#include "CertificateVerifier.hpp"
#include "ExternalEvaluator.hpp"
#include "URL.hpp"
#include "CertInfoCollector.hpp"

namespace Akenti {

class RuntimePolicy;

/**
  *
  * @see CertificateCache 
  * @see CertificateVerifier 
  * 
  * @version 1.1 00/05/01
  * @author Abdelilah Essiari
  **/
class AKRuntime {
  
public:
   
  /**
    */
  ~AKRuntime();

  /**
    * Returns the name of the resource.
    *
    * @see PolicyContext#getResource()
    */
  const string& getResource();

  /**
    * Returns the user.
    *
    * @see PolicyContext#getAkentiPrincipal()
    */
  const AkentiPrincipal& getAkentiPrincipal();

  /**
    * Returns the policies.
    *
    * @see Policy
    * @see PolicyContext#getPolicies()
    */
  const vector<Policy>& getPolicies();

  /**
    * Returns the policy context.
    *
    * @see PolicyContext
    */
  const PolicyContext& getPolicyContext();

  /**
    * Returns the  CertInfo collector
    *
    * @see CertInfoCollector
    */
  CertInfoCollector& getCertInfoCollector();

  /**
    * Returns the desired actions.
    *
    * empty => all actions are desired.
    * @see PolicyContext#getDesiredActions();
    */
  const set<string>& getDesiredActions();

  /**
    * Returns the desired attributes.
    *
    * @see PolicyContext#getRestrictedAttributes()
    */
  const multimap<string, string>& getRestrictedAttributes();

  /**
    * Returns true if action is desired.
    *
    * Note: empty ==> all actions are desired.
    */
  bool isAttributeDesired(const string& attribute, const string& value) const;

  /**
    * Returns true if action is desired.
    */
  bool isActionDesired(const string& action) const;

  /**
    * Returns the urls used to locate attribute directories.
    */
  const vector<URL>& getAttributeURLS();

  /**
    * Returns the certificate verifier.
    *
    * @see CertificateVerifier
    */
  CertificateVerifier& getCertificateVerifier();

  /**
    * Returns the cache interval.
    */
  long getCacheInterval();

  /**
    * Use this to cache certificates
    */
  CertificateCache& getCertificateCache();

  /**
    */
  void setResource(const string& resource);

  /**
    */
  void setAkentiPrincipal(const AkentiPrincipal& ap);

  /**
    */
  void setResourceAltName(const string& resAltName);

  /**
    */
  void setSubjectCert(const AkentiX509Certificate& cert);

  /**
    */
  void setCapabilityCert(const CapabilityCertificate& cert);

  /**
    */
  void addAkentiX509Cert(const AkentiX509Certificate& x509Cert);

  /**
    */
  void addAttributeCert(const AttributeCertificate& attrCert);

  /**
    */
  void addUnconditionalAction(const string& unconditionalAction);

  /**
    */
  void addConditionalActions(const ConditionalActions& condActions);

  /**
    */
  const vector<AKCertHolder>& getCACerts(int startLevel, int &caLevel);

  /**
    */
  int getCALevel(int level);

  /**
    */
  const vector<URL>& getIdentityURLS(int level);

  /**
    */
  void addPolicy(const Policy& policy);

  /**
    */
  Policy& getPolicy(int index);

  /**
    */
  int numOfPolicies();

  /**
    */
  void setLevel(int level);

  /**
    */
  int getLevel();

  /**
    */
  void setExternalEvaluator(const ExternalEvaluator& externalEvaluator);

  /**
    */
  void setDesiredActions(const set<string>& desiredActions);

  /**
    */
  void setRestrictedAttributes(const multimap<string, string>& desiredAttributes);

  /**
    */
  void setCacheCap(bool);

  /**
    */
  bool getCacheCap() const;

  /**
    */
  ExternalEvaluator& getExternalEvaluator();

  /**
    */
  bool shouldStop() const;

  /**
    */
  string paramString() const;

  /**
    */
  static AKRuntime* createAKRuntime();

private:

  AKRuntime(bool stop = true);

  int level;

  CertificateCache certCache;

  CertificateVerifier certVerifier;
  
  vector<RuntimePolicy*> runtimePolicies;
  
  PolicyContext ctx;
  
  CertInfoCollector cinfoCol;
  
  ExternalEvaluator externalEvaluator;
  
  bool stop;

  bool cacheCap;
};

};  // end of namesapce

#endif
