#ifndef CERT_INFO_HPP
#define CERT_INFO_HPP

#include <string>

#include "AkentiCertificate.hpp"
#include "AkentiMessage.hpp"

using std::string;

using namespace Akenti;

namespace Akenti {

/** 
  * Class <code>CertInfo</code> contains all the information pertaining 
  * to one  <i>Policy</i> or <i>Use Condition</i> certificate and whether 
  * it was used or not in determining whether to provide access to a user.
  *
  * @type CertInfo
  *
  * @see AkentiCertificate
  * @see AkentiMessage
  *
  * @version 1.0 04/27/04
  * @author Sushila R. Subramanian
  *
  **/

class CertInfo {
public:

   CertInfo();

  /** 
    * Destroys this object.
    */
  ~CertInfo ();

  /** 
    * Constructor for dummy certificate cases 
    *
    * @see AkentiCertificate
    * @see AkentiMessage
    */

   CertInfo (bool Used, AkentiCertificate cert, AkentiMessage mesg); 

  /** 
    * Constructor for known <i>Policy</i> and <i>Use Condition</i> certificates 
    * 
    * @see AkentiCertificate
    * @see AkentiMessage
    */

   CertInfo (bool Used, AkentiCertificate cert, AkentiMessage mesg,
       string Hash, string Type, string Resource); 

  /** 
    * Constructor for known <i>Policy</i> and <i>Use Condition</i> certificates 
    * 
    * @see AkentiCertificate
    * @see AkentiMessage
    */

   CertInfo (bool Used, AkentiCertificate cert, AkentiMessage mesg,
       string Hash, string Type, string Resource, bool err); 

  /**
    * Add AkentiMessage associated with collecting or verifying this certificate
    *
    * @see AkentiMessage
    */
   void addMessage (AkentiMessage mesg);

  /**
    * Add the certificate for which you are keeping usage status
    *
    * @see AkentiCertificate
    */
   void addCert (AkentiCertificate cert);

   /**
    *  Returns the status messages associated with this certificate
    *
    *  @see AkentiMessage 
    */

   const vector <AkentiMessage>& getMessages ();

   /**
    *  Returns the Akenti certificate 
    *
    *  @see AkentiCertificate
    */

   AkentiCertificate getCertificate ();

   /**
     * Sets the used boolean if this certificate is used in determining
     * if the Policy is valid.
     */

   void setUsed (bool usedVal);

   /** 
    *  Returns whether this Certificate was used or not.
    */

   bool getUsed ();

   /**
     * Sets the error boolean if there was an error in this certificate
     */

   void setError (bool err);

   /** 
    *  Returns whether this Certificate had an error or not
    */

   bool getError ();

   /**
    *  Used by PolicyCertificates only - to determine if Use conditions are
    *  used.
    */

   void setLeaf (bool leafVal);

   /** 
    */

   bool isLeaf ();

   /** 
     * Set the hashcode associated with the certificate. If its a 
     * dummy certificate - set it to "dummy"
     */ 

   void setHashCode (string hcode);

   /**
     * Returns Hashcode (used in looking up cert)
     */
   string getHashCode ();

   /** 
     * Set the resource associated with this cert. If dummy or no
     * resource - set it to "NO_RESOURCE".
     */

   void setResource (string res);

   /**
     * Get resource 
     */
   string getResource ();

   /** 
     * Set the type associated with this cert. If dummy 
     * - set it to "dummy".
     */

   void setType (string ctype);

   /**
     * Get type 
     */
   string getType ();

   /** 
     * Set the location of the certificate (string containing URL form)
     * Only makes sense for UC Certificates
     */

   void setLocation (string slocation);

   /**
     * Get location where cert was obtained
     * Only makes sense for UC Certificates
     */
   string getLocation ();

  /**
    * For debugging. - not written yet
    */

   /* string paramString() const; */

private:

   bool  used;  // default set to true; assume used unless error
 
   bool  error; // default set to false; true means there is an error 

   vector <AkentiMessage> statusMsgs; // set to OK or the error    
  
   AkentiCertificate  akCert; // akenti certificate

   string resource;
  
   string hashCode; 
  
   string type; // Policy, Use Condition, or dummy
 
   bool leaf;

   string location; 

}; // end class 

} // end of name space 
#endif /* CERT_INFO_HPP */
