#ifndef AK_FILE_HPP
#define AK_FILE_HPP
#include <string>
#include <vector>

using std::string;
using std::vector;

/**
  * A <i>File<i> object contains a  filename and provides methods to return
  * informataion about the name or directory. The filename may be either 
  * an absolute pathname or relative to the current working directory. It
  * may refer to either a plain file or a directory.
  *
  * <PRE>
  * <b>Usage:</b>
  *  eg 1). File("Makefile");    
  *  eg 2). File("/usr/local/mrt/Certificates");    
  * </PRE>
  * 
  * @see Filter
  *
  * @version 1.1 00/11/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

namespace Akenti {

class Filter;

class File {

public:

  /**
    * Construct this object. If filename starts with "/" it
    * is an absolute pathname, otherwise it is relative to the current
    * working directory.
    *   
    * @param filename string name of the file,
    */
  File(const string& filename);

  /**
    * Gets the name of the file.
    *
    * @return name of the file
    */
  string getName() const;

  /**
    * Gets the absolute path of the file.
    *
    * @return string the absolute path
    * @exception Error if we cannot determine the working directory.
    */
  string getAbsolutePath() const;

  /**
    * Gets the Parent directory of the file. If the filename is a single "/"
    * an empty string is returned. If the name is relative (contains no "/")
    * an empty string is returned.
    *
    * @return string the parent name of the file
    */
  string getParent() const;

  /**
    * Checks to see if the file has parent directory.
    *
    * @return bool an indication if the file has a parent or not
    */
  bool hasParent() const;

  /**
    * Checks to see if the filename is an absolute path form (i.e. starts with
    * a "/".
    *
    * @return bool true if the file is absolute, otherwise false.
    */
  bool isAbsolute() const ;

  /**
    * Checks to see if the file exists or not. 
    *
    * @return bool true if the file exists, otherwise false.
    */
  bool exists() const;

  /**
    * Checks to see if the file exists and is a directory. 
    *
    * @return bool true if the file is a directory, otherwise false.
    */
  bool isDirectory() const;

  /**
    * Checks to see if the file exists and is not a directory
    *
    * @return bool true if the file is not a directory, other false.
    */
  bool isFile() const;
  
  /**
    * Returns a list of the contents of this directory. 
    * 
    * @return vector<string> list of all the directory entries
    * @exception Error: file does not exist
    * @exception Error: file is not a directory
    * @exception IOException: unable to open directory
    */
  vector<string> list() const;

  /**
    * Returns a list of file names after being filtered. 
    * 
    * @param filter a pattern that a name must match in order to be included
    *        in the returned list of names.
    * @return vector<string> collection of filtered file name(s)
    * @exception Error if file does not exits or 
    * @exception Error file is not a directory
    * @exception IOException if unable to open directory 
    */
  vector<string> list(const Filter& filter) const;

  /**
    * Create a directory.
    *
    * @exception Error 
    * @exception IOException
    */
  void mkdir() const;
  
  /**
    * Deletes this <i>file/i>. 
    *
    * A directory File must be empty for this call to succeed. 
    *
    * @return true if succesful 
    *         false otherwise.
    */
  bool remove() const;

  /**
    * Returns a string representation for File which is used for 
    * debugging purposes.
    *
    * @return debugging info
    */
  string paramString() const;

  /**
    * For logging purpose.
    */
  string toLogString() const;

private:

  string filename;

};

} // end namespace Akenti

#endif
