#ifndef POLICY_HPP
#define POLICY_HPP

#include <vector>
#include "PolicyCertificate.hpp"
#include "UseCondCertificate.hpp"

using std::vector;
using namespace Akenti;

namespace Akenti {

class Directory;
class UseCondIssuerGroup;
class CertificateAuthorityInfo;

class PolicyImpl;

/**
  * Class <code>Policy</code> serves as the wrapper class
  * for <code>PolicyImpl</code> objects. It has a
  * <i>reference-count</i> for the implementation object that it holds.
  *
  * @type Policy
  *
  * @see PolicyImpl
  */
typedef ref_ptr<PolicyImpl> Policy;

/**
  * Wraps a policy cert and is used to store <i>UseCondition</i> 
  * certs that apply to the policy cert's resource.
  *
  * @see AkentiCertificate
  * @see AkentiIssuer
  * @see CertificateAuthorityInfo 
  * @see Directory 
  * @see UseCondIssuerGroup 
  * 
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

class PolicyImpl {

public:

  /**
    * Constructs this object. 
    *
    * The <i>policyCert</i> must be of type <i>Policy</i>.
    *
    * @param policyCert policy certificate
    *
    * @see AkentiCertificate
    */
  PolicyImpl(bool leaf, const PolicyCertificate& policyCert);

  /** 
    * Destroys this object.
    */
  ~PolicyImpl();

  /**
    */
  bool isLeaf() const;

  /**
    */
  const PolicyCertificate& operator->() const;

  /**
    * Returns the name of the resource.
    *
    * @see PolicyCertificateImpl#getResource()
    */
  const string& getResource() const;

  /**
    * Returns the hashCode of the policy certificate.
    *
    * @see AkentiCertificateImpl#hashCode()
    */
  string hashCode() const;

  /** 
    * Returns the <i>cacheTime</i> in seconds. 
    *
    * @see PolicyCertificateImpl#cacheInterval()
    */
  long getCacheInterval() const;

  /** 
    * Returns the <i>AkentiIssuer</i> of the policy certificate.
    *
    * @see AkentiCertificateImpl#getIssuer()
    * @see AkentiIssuer
    */
  const AkentiIssuer& getIssuer() const;

  /**
    * Returns the policy certificate.
    *
    * @see AkentiCertificate
    */
  const PolicyCertificate& getPolicyCertificate() const;

  /**
    * Returns the </i>CertificateAuthorityInfo</i>s.
    *
    * @see PolicyCertificateImpl#getCertificateAuthorityInfos()
    * @see CertificateAuthorityInfo
    */
  const vector<CertificateAuthorityInfo>& 
  getCertificateAuthorityInfos() const;

  /**
    * Returns the attribute directories.
    *
    * @see PolicyCertificateImpl#getAttributeDirectories()
    * @see Directory
    */
  const vector<Directory>& getAttributeDirectories() const;

  /**
    * Returns the number of <i>UseCondIssuerGroup</i>s.
    *
    * @see PolicyCertificateImpl#getUseCondIssuerGroups()
    */
  int numOfUseCondIssuerGroups() const;

  /**
    * Returns the <i>UseCondIssuerGroup</i> at <i>groupIndex</i>.
    *
    */
  const UseCondIssuerGroup& 
  getUseCondIssuerGroup(int groupIndex) const;

  /**
    * Adds a <i>UseCondition</i> certificate.
    *
    * @see UseConditionCertificateImpl
    **/
  void addUCCertificate(const UseCondCertificate& ucCert);

  /**
    * Returns all <i>UseCondition</i> certificates.
    *
    * @see UseConditionCertificateImpl
    */
  const vector<UseCondCertificate>& getUCCerts() const;

  /**
    * Returns critical <i>UseCondition</i> certificate 
    */
  vector<UseCondCertificate> getCriticalUCCerts() const;

  /**
    * Returns flexible <i>UseCondition</i> certificates. 
    */
  vector<UseCondCertificate> getFlexibleUCCerts() const;

  /**
    * For debugging purposes.
    */
  string paramString() const;

  /**
    * For logging purposes.
    */
  string toLogString() const;

private:

  bool leaf;

  PolicyCertificate policyCert;         // the policy certificate

  vector<UseCondCertificate> ucCerts;   // the UseCondition certificates

};

}; // end of name space

#endif
