#ifndef AK_SSL_CONTEXT_IMPL_HPP
#define AK_SSL_CONTEXT_IMPL_HPP

#include <sstream>
#include "SSLConstants.hpp"
#include "Error.hpp"
#include "Certificate.hpp"

using std::ostringstream;

namespace Akenti {

/**
  * Interface for security contexts that mainly hold credentials and trusted ca infos.
  * 
  * @see SSLConstants
  * @see Version
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari  
  * 
  */

class SSLContextImpl : public SSLConstants {

public:

  /**
    * Destructor.
    */
  virtual ~SSLContextImpl() {
  }

  /**
    * Returns the version number of the security context provider.
    */
  Version getVersion() const {
    return version;
  }

  /**
    * Returns true if this is a client context.
    */
  bool isClientContext() {
    return isClient;
  }

  /**
    * Sets the level of peer verfication that is required.
    *
    * <pre>
    *   @param verify level. The choices for client contexts are:
    *          VERIFY_NONE
    *          VERIFY_PEER
    *          VERIFY_PEER | VERIFY_FAIL_IF_NO_PEER_CERT
    *     The following choices are also available for server contexts.
    *          VERIFY_PEER | VERIFY_CLIENT_ONCE
    *          VERIFY_PEER | VERIFY_FAIL_IF_NO_PEER_CERT | VERIFY_CLIENT_ONCE
    *  </pre>
    */
  void setVerify(int verify) {
    switch(verify) {
      case VERIFY_PEER | VERIFY_CLIENT_ONCE:
      case VERIFY_PEER | VERIFY_FAIL_IF_NO_PEER_CERT | VERIFY_CLIENT_ONCE:
        if (isClientContext()) {
          throw Error("SSLContextImpl:setVerify:invalid arg(1)");
        }

      case VERIFY_NONE:
      case VERIFY_PEER:
      case VERIFY_PEER | VERIFY_FAIL_IF_NO_PEER_CERT:
        doSetVerify(verify);
        break;
      default:
        throw Error("SSLContextImpl:setVerify:invalid arg(2)");
    }
  }
  
  /**
    * Sets the path name of the file that contains the principal's 
    * X.509 identity certificate.
    *
    * @param certFile  Pathname of the file containing the identity cert.
    * @param type  either PEM or ASN1
    */
  virtual void setCertificateFile(const string& certFile, FileType type = SSLConstants::PEM) = 0;

  /**
    * Sets the path name of the file that contains the principal's private key
    *
    * @param keyFile  Pathname of the file containing the private key.
    * @param type  either PEM or ASN1
    */
  virtual void setPrivateKeyFile(const string& keyFile, FileType type = SSLConstants::PEM) = 0;

  /**
    * Adds a trusted ca.
    */
  virtual void addTrustedCA(const Certificate& cert) = 0;

  /**
    * Sets the trusted cas. 
    *
    * @param name  name of a file or directrory
    */ 
  virtual void setTrustedCAS(const string& filename)  = 0;

  /**
    * Returns debugging info.
    */
  virtual string paramString() const {
    ostringstream os;
    
    os << '['   
       << "version" 
       << '=' 
       << getVersion() 
       << ','
       << "isClient"
       << '=' 
       << isClient   
       << ']';  

    return os.str(); 
  }   
     

protected:

  SSLContextImpl(Version version, bool isClient) 
    : version(version), isClient(isClient) {
  }

  virtual void doSetVerify(int verify) = 0;

private:
  
  Version version;

  bool isClient;

};

} // end of namespace Akenti

#endif
